#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/ip.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <netinet/if_ether.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <errno.h>

#define IPVERSION 4
#define DEFAULT_TTL 255
#define IPHEADER_LENGTH 5
#define IPPROTOCOL 255

extern int AliceBDhashCreator( unsigned char *mac, unsigned char *hash );

unsigned short ip_cksum(unsigned short *address, int length)
{
    register int sum = 0;
    u_short answer = 0;
    register u_short *w = address;
    register int nleft = length;

    while (nleft > 1) {
	sum += *w++;
	nleft -= 2;
    }

    if (nleft == 1) {
	*( u_char * )( &answer ) = *( u_char * )w ;
	sum += answer;
    }

    sum = (sum >> 16) + (sum & 0xffff); 
    sum += (sum >> 16);
    answer = ~sum;
     
    return(answer);
}

int ip_gen(char *packet, unsigned char protocol, struct in_addr saddr, struct in_addr daddr, unsigned short length)
{
    struct iphdr *ipheader;
     
    ipheader = (struct iphdr *) packet;
    memset((char *) ipheader, 0, sizeof(struct iphdr));

    // Riempimento della struttura ip
    ipheader->ihl = IPHEADER_LENGTH; 
    ipheader->version = IPVERSION;
    // Endianness
#ifdef IP_LEN_HORDER
    ipheader->tot_len = length;
#else
    ipheader->tot_len = htons(length);
#endif

    ipheader->id = 0;
    ipheader->ttl = DEFAULT_TTL;
    ipheader->protocol = protocol;
    ipheader->saddr = saddr.s_addr;
    ipheader->daddr = daddr.s_addr; 

    // Calcolo del checksum
    ipheader->check = (unsigned short) ip_cksum((unsigned short *) ipheader, sizeof(struct iphdr));

    return 0;
}

int hex2byte(unsigned char *output, char *input)
{
    int iRet = 0;
    unsigned long ulRes;

    ulRes = strtoul(input, 0, 16);
    if (errno == EINVAL || errno == ERANGE) {
	iRet = -1;
    } else if (ulRes >= 256) {
	iRet = -2;
    } else {    
	*output = (unsigned char) ulRes;
    }
    
    return iRet;
}

int main(int argc, char *argv[])
{
    // Array di byte che conterr il pacchetto
    // lungo ip header + 8
    unsigned char packet[ sizeof(struct iphdr) + 8 ];
    struct sockaddr_in sa_socket;
    struct in_addr srcaddr, dstaddr;

    int sockd;
    int one = 1;

    struct ifreq ifreq;
    char str2[3];
    unsigned char ucMac[6];
    unsigned char ucHash[8];
    int i;
    
    // Gestione argomenti
    if(argc < 4) {
	printf("Usage: %s local_interface router_mac_address router_ip_address\n", argv[0]);
	exit(-1);
    }

    // Traduzione mac address da stringa a bytearray
    for (i = 0; i < 18; i = i + 3) {
	str2[0] = argv[2][i];
	str2[1] = argv[2][i+1];
	str2[2] = '\0';
	if (hex2byte(&ucMac[i/3], str2) < 0) {
    	    perror("socket");
    	    exit(-2);
	}
    }
    
//    printf("%02x - %02x - %02x - %02x - %02x - %02x\n", ucMac[0], ucMac[1], ucMac[2], ucMac[3], ucMac[4], ucMac[5]);

    // Creazione hash
    AliceBDhashCreator(ucMac, ucHash);

    // Creazione del socket descriptor
    if((sockd = socket(AF_INET, SOCK_RAW, IPPROTO_RAW)) < 0) {
        perror("socket");
        exit(-3);
    }

    if(setsockopt(sockd, IPPROTO_IP, IP_HDRINCL, &one, sizeof(one)) < 0) {
        perror("setsockopt");
        exit(-4);
    }

    strcpy (ifreq.ifr_name, argv[1]);
    if (ioctl (sockd, SIOCGIFADDR, &ifreq) < 0) {
	perror ("SIOCGIFADDR");
        exit (-5);
    }

    memcpy (&srcaddr, &((struct sockaddr_in *) &ifreq.ifr_ifru.ifru_addr)->sin_addr, sizeof (struct sockaddr_in));

    dstaddr.s_addr = inet_addr(argv[3]);

    // Creazione pacchetto IP
    ip_gen(packet, IPPROTOCOL, srcaddr, dstaddr, sizeof(packet));
    memcpy(&packet[20], ucHash, 8);
    
    memset(&sa_socket, '\0', sizeof(sa_socket));

    sa_socket.sin_family = AF_INET;
    sa_socket.sin_port = 0;
    sa_socket.sin_addr = dstaddr;

    // Invio pacchetto
    if(sendto(sockd, &packet, sizeof(packet), 0x0, (struct sockaddr *) &sa_socket, sizeof(sa_socket)) != sizeof(packet)) {
	perror("sendto");
	exit(-6);
    } 
    
    return 0;
}
